classdef OCPFields
    %OCPFields Utility Class to convert between API and Server field names
    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Copyright 2015 The Johns Hopkins University / Applied Physics Laboratory 
    % All Rights Reserved. 
    % Contact the JHU/APL Office of Technology Transfer for any additional rights.  
    % www.jhuapl.edu/ott
    %  
    % Licensed under the Apache License, Version 2.0 (the "License");
    % you may not use this file except in compliance with the License.
    % You may obtain a copy of the License at
    %  
    %     http://www.apache.org/licenses/LICENSE-2.0
    %  
    % Unless required by applicable law or agreed to in writing, software
    % distributed under the License is distributed on an "AS IS" BASIS,
    % WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    % See the License for the specific language governing permissions and
    % limitations under the License.
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    properties (SetAccess = 'private', GetAccess = 'public')
        generic
        seed
        synapse
        segment
        organelle
        neuron        
    end
    
    methods
        function this = OCPFields()
           % Generic 
           this.generic = struct('status','status',...
                                'confidence','confidence',...
                                'author','author');
                            
           % seed 
           this.seed = struct('status','status',...
                                'confidence','confidence',...
                                'author','author',...
                                'cubeOrientation','cubelocation',...
                                'parentSeed','parent',...
                                'sourceEntity','source',...
                                'position','position');
                            
           % synapse 
           this.synapse = struct('status','status',...
                                'confidence','confidence',...
                                'author','author',...
                                'synapseType','synapse_type',...
                                'weight','weight',...
                                'segments','segments',...
                                'seeds','seeds');
                            
           % segment 
           this.segment = struct('status','status',...
                                'confidence','confidence',...
                                'author','author',...
                                'class','segmentclass',...
                                'neuron','neuron',...
                                'synapses','synapses',...
                                'organelles','organelles',...
                                'parentSeed','parentseed');
           
           % neuron 
           this.neuron = struct('status','status',...
                                'confidence','confidence',...
                                'author','author',...
                                'segments','segments');
                            
           % neuron 
           this.organelle = struct('status','status',...
                                'confidence','confidence',...
                                'author','author',...
                                'parentSeed','parentseed',...
                                'class','organelleclass',...
                                'seeds','seeds');
        end
        
    end
    
end

